/*

Atmel Prozessor ATMEGA 32 bei 1 MHz

Frequenzumrichter mit Raumzeigermodulation
Steuerungsprozessor 
U/f Betrieb

Ein- und Ausgnge

PORT A 0 (40) = Analogwert Uzk   (ADC0 ==> ADMUX=0)
PORT A 1 (39) = Analogwert Strom Zk (ADC1 ==> ADMUX=1)
PORT A 2 (38) = Klemme 2 (ADC2 ==> ADMUX=2) Frequenzsollwert
PORT A 3 (37) = Klemme 3
PORT A 4 (36) = Klemme 4 interner Pull up
PORT A 5 (35) = Klemme 5 mit Pull Up Widerstand Takt von Drehzahlerfassung
PORT A 6 (34) = Klemme 6 mit Pull Up Widerstand 
PORT A 7 (33) = Klemme 7 mit Pull Up Widerstand 

PORT B 0 ( 1) = LCD PORT 1 11	DB4
PORT B 1 ( 2) = LCD PORT 2 12	DB5
PORT B 2 ( 3) = LCD PORT 3 13	DB6
PORT B 3 ( 4) = LCD PORT 4 14	DB7
PORT B 4 ( 5) = LCD PORT 5 4	RS
PORT B 5 ( 6) = LCD PORT 6 6	E
PORT B 6 ( 7) = LCD PORT 7
PORT B 7 ( 8) = LCD PORT 8

PORT C 0 (22) = (Enable) (PC5 an Motorprozessor)
PORT C 1 (23) = Spannung (PC4 an Motorprozessor)
PORT C 2 (24) = Spannung (PC3 an Motorprozessor)
PORT C 3 (25) = Spannung (PC2 an Motorprozessor)
PORT C 4 (26) = Spannung (PC1 an Motorprozessor)
PORT C 5 (27) = Spannung (PC0 an Motorprozessor)

PORT C 6 (28) = Klemme 8 mit Pull Up Widerstand (Enable)

PORT C 7 (29) = frei

PORT D 0 (14) = Frequenz (PD6 an Motorprozessor)
PORT D 1 (15) = Frequenz (PD5 an Motorprozessor)
PORT D 2 (16) = Frequenz (PD4 an Motorprozessor)
PORT D 3 (17) = Frequenz (PD3 an Motorprozessor)
PORT D 4 (18) = Frequenz (PD2 an Motorprozessor)
PORT D 5 (19) = Frequenz (PD1 an Motorprozessor)
PORT D 6 (20) = Frequenz (PD0 an Motorprozessor)

PORT D 7 (21) = Richtung (PB2 an Motorprozessor)



LCD Module    Anschluss

1	Vss	GND
2	Vcc	5V
3	Vee	GND oder Poti 
4	RS	PB4 am AVR
5	RW	GND
6	E	PB5 am AVR
7	DB0	GND
8	DB1	GND
9	DB2	GND
10	DB3	GND
11	DB4	PB0 am AVR
12	DB5	PB1 am AVR
13	DB6	PB2 am AVR
14	DB7	PB3 am AVR

*/




#include <inttypes.h>
#include <avr/io.h>

// #include <avr/signal.h>

#include <avr/interrupt.h>

// Standard Input/Output functions
#include <stdio.h>


volatile signed char sollvh=0; 	// sollnh Sollwert vor Hochlaufgeber
volatile signed char sollnh=0; 	// sollnh Sollwert nach Hochlaufgeber
volatile signed char sollabs=0; 	// Sollwert absolut
volatile unsigned char richtung=0; 	// Drehrichtung
volatile unsigned char hlg=10;			// Hochlaufgeber

volatile unsigned char display[33];	// Displaypeicher
volatile unsigned char dispcount=1;			// Display Counter	

volatile unsigned char nist=0;
volatile unsigned char nistsample=0;



/************************************************************************* 
delay for a minimum of <us> microseconds
with a 4Mhz crystal, the resolution is 1 us / 16Mhz crystal, the resolution is 0,25 s
*************************************************************************/
static void delay(uint16_t us) 
{
    while ( us ) us--;
}



// ******************************************************************* 
//                        LCD e toggle        
// ******************************************************************* 
void lcd_e_toggle(void)
{    
PORTB |= _BV(5); 		//Enable high
	delay(1);			// 1s 
PORTB &=~_BV(5) ;		//Enable wieder low
}




// ******************************************************************* 
//                        Ascii Zeichen anzeigen  ohne DELAY  
// ******************************************************************* 
void lcd_putcho(uint8_t data)		//;wie lcd_command, nur mit RS setzen
{
unsigned char temp;
	temp=data>>4;
	temp=temp|0x10;
	PORTB=temp;
	lcd_e_toggle();
	temp=data & 0x0F;
	temp=temp|0x10;
	PORTB=temp;
	lcd_e_toggle();

}


// ******************************************************************* 
//                        LCD Steuerung / Kommandos    
// ******************************************************************* 
void lcd_command(uint8_t cmd)		//;wie lcd_putch, nur ohne RS zu setzen
{

unsigned char temp;

	temp=cmd>>4;
	PORTB=temp;
	lcd_e_toggle();
	temp=cmd & 0x0F;
	PORTB=temp;
	lcd_e_toggle();
	delay(40);		// 750 s bei 1 MHz


}




/*-----------------27.5.99 20:31--------------------
 *  Goto position   (0x00 - 1.row, 1.col)
 *                  (0x40 - 1.row, 2.col)
 *--------------------------------------------------*/
void lcd_goto(unsigned char mesto)
{
  unsigned char temp;
  temp=0x08;
  temp|=mesto>>4;
  PORTB=temp;
  lcd_e_toggle();
  PORTB=mesto & 0x0F;
  lcd_e_toggle();
  delay(70);	// 1250s bei 1 Mhz
}



// ******************************************************************* 	
void lcd_init(void)
{
	DDRB = 0xFF;		// all port bits as output 
	delay(6400);		// wait 25ms or more after power-on bei 10 MHz
	delay(6400);		// wait 25ms or more after power-on bei 10 MHz    
   
 
	PORTB=0x03; lcd_e_toggle(); delay(300); 	// 5000s bei 1 MHz
	PORTB=0x03; lcd_e_toggle(); delay(300); 	// 5000s bei 1 MHz
	PORTB=0x03; lcd_e_toggle(); delay(300); 	// 5000s bei 10 MHz
	
	PORTB=0x02; lcd_e_toggle(); delay(180);	// 2500 s bei 1 Mhz
	
	lcd_command(0x28);	// 0010 1000
	lcd_command(0x0c);	// 0000 1100
	lcd_command(0x04);	// 0000 0100	

}/* lcd_init */
    	






/* ******************************************************************* 
                      Timerinterrupt 0  

TCCR0 = 3; => Takt / 64   
TCCR0 = 4; => Takt / 256  
TCCR0 = 5; => Takt / 1024 
TCCR0 = 5; => Takt / 1024 


	Hoch- / Rcklaufgeber
      hlg = 20 ==> 10,15 sec Hochlaufzeit 0 bis 50 Hz
      
                    
                    
 ******************************************************************* 
*/

SIGNAL(SIG_OVERFLOW0)            // signal handler for tcnt0 overflow interrupt
{				// alle 10,24ms    
 
hlg++;
if (hlg >=10) {

		if (sollvh > sollnh) sollnh++;
		if (sollvh < sollnh) sollnh--;
		hlg=0;
		}

TCNT0 = 246;  // Timer auf 10,24ms stellen







// Da die LCD Ausgabe mit den blichen Wartezeiten sehr viel Zykluszeit frisst, 
// wird im Hauptprogramm nur ein Displayspeicher beschrieben.
// In dieser Timerroutine wird pro Durchlauf ein Zeichen auf dem LCD ausgegeben.


// ************** LCD Ausgabe **************
lcd_putcho(display[dispcount]);
dispcount++;
if (dispcount==17) lcd_goto(0x40);		// Wechsel in die 2. Zeile Position 1 (Anfang)
if (dispcount==33) {lcd_goto(0x00);dispcount=1;}
// *****************************************************



}

// Ist-Drehzahlerfassung noch im Test!!!!!!!!!!!!!!!!!!!!!!!!!!!
// ******************************************************************* 
//                       Timerinterrupt 1  
//	999,94ms                     
// ******************************************************************* 
SIGNAL(SIG_OVERFLOW1)            // signal handler for tcnt1 overflow interrupt
{

nist=nistsample;
nistsample=0;



TCNT1H=0xf0;  
TCNT1L=0xbe;

}


// ******************************************************************* 
//                       Initialisierung          
// ******************************************************************* 
void initial(void)
{    

TCNT0 = 0;        	//  TCNT0 Timer auf ??s einstellen 
TCCR0 = 5;        	// count with cpu clock/1024

TCCR1A =0;          	// disable PWM and Compare Output Mode
TCCR1B=4;  		// use CLK/1024 prescale value (entspricht 256 sec bei 4 MHz)
TCNT1H=0xff; 		// reset TCNT1 high-byte first 
TCNT1L=0xff;  		// reset TCNT1 low
 
TIMSK = (1<<TOIE0|1<<TOIE1); 	// enable TCNT1 overflow

sei();			// enable interrupts


DDRA = 0;	// PortA   ( alles Eingnge)
PORTA= 0xF0;
DDRC = 0x3f;	// PortC0-5 Ausgnge PC6 = Eingang
DDRD = 0xff;      	// PortD   ( alles Eingnge)

 
ADCSRA=0x84;	// ADEN =1 / Prescaler = 16


}


// ******************************************************************* 
//                       Hauptprogramm          
// ******************************************************************* 

void main(void)
{ 

// ******************************************************************* 
//              Variablen fr das Hauptprogramm       
// ******************************************************************* 


unsigned int i = 0;
unsigned char j = 0;
unsigned int y=0;
unsigned char spg=0;
unsigned char gespiegelt=0;

// unsigned int sample=0;
unsigned int strom_ist=0;
unsigned int strom_ist_sample=0;
unsigned int spannung_ist=0;
unsigned int spannung_ist_sample=0;
unsigned int sollwert_f=0;
unsigned int sollwert_f_sample=0;
unsigned int sollwert_u=0;
unsigned int sollwert_u_sample=0;


unsigned char ntakt=0;
unsigned char ntaktalt=0;

unsigned char enable=0;


// Spannungssollwert, 20 entspricht 100% (wird bei 57 Hz erreicht)
const unsigned char spannung[81]={
0	,     // 0   Hz	0
1	,     // 1   Hz	1
1	,     // 2   Hz	2
1	,     // 3   Hz	3
1	,     // 4   Hz	4
2	,     // 5   Hz	5
2	,     // 6   Hz	6
2	,     // 7   Hz	7
3	,     // 8   Hz	8
3	,     // 9   Hz	9
4	,     // 10   Hz	10
4	,     // 11   Hz	11
4	,     // 12   Hz	12
5	,     // 13   Hz	13
5	,     // 14   Hz	14
5	,     // 15   Hz	15
6	,     // 16   Hz	16
6	,     // 17   Hz	17
6	,     // 18   Hz	18
7	,     // 19   Hz	19
7	,     // 20   Hz	20
7	,     // 21   Hz	21
8	,     // 22   Hz	22
8	,     // 23   Hz	23
8	,     // 24   Hz	24
9	,     // 25   Hz	25
9	,     // 26   Hz	26
9	,     // 27   Hz	27
10	,     // 28   Hz	28
10	 ,    // 29   Hz	29
11	,     // 30   Hz	30
11	,     // 31   Hz	31
11	,    // 32   Hz	32
12	 ,    // 33   Hz	33
12	,     // 34   Hz	34
12	,     // 35   Hz	35
13	,     // 36   Hz	36
13	 ,    // 37   Hz	37
13	,     // 38   Hz	38
14	 ,    // 39   Hz	39
14	,     // 40   Hz	40
14	,     // 41   Hz	41
15	,     // 42   Hz	42
15	,    // 43   Hz	43
15	,     // 44   Hz	44
16	,     // 45   Hz	45
16	,     // 46   Hz	46
16	,     // 47   Hz	47
17	,     // 48   Hz	48
17	,     // 49   Hz	49
18	,     // 50   Hz	50
18	 ,    // 51   Hz	51
18	,     // 52   Hz	52
19	,     // 53   Hz	53
19	,     // 54   Hz	54
19	,     // 55   Hz	55
20	,     // 56   Hz	56
20	,     // 57   Hz	57
20	,     // 58   Hz	58
20	,     // 58   Hz	59
20	,     // 58   Hz	60
20	,     // 58   Hz	61
20	,     // 58   Hz	62
20	,     // 58   Hz	63
20	,     // 58   Hz	64
20	,     // 58   Hz	65
20	,     // 58   Hz	66
20	,     // 58   Hz	67
20	,     // 58   Hz	68
20	,     // 58   Hz	69
20	,     // 58   Hz	70
20	,     // 58   Hz	71
20	,     // 58   Hz	72
20	,     // 58   Hz	73
20	,     // 58   Hz	74
20	,     // 58   Hz	75
20	,     // 58   Hz	76
20	,     // 58   Hz	77
20	,     // 58   Hz	78
20	,     // 58   Hz	79
20	,     // 58   Hz	80



   };

// ******************************************************************* 

lcd_init();	// LCD Initialisierung
lcd_command(0x01);	// LCD Lschen
delay(90);		// 1250s warten bei 1 Mhz


initial();

for(i=0;i<33;i++)  
    {
     display[i] = 0x20;	// Displayspeicher lschen
    }

display[1] = 0x53;  //S=
display[2] = 0x3d;

display[5] = 0x55;	//U=
display[6] = 0x3d;

display[9] = 0x52;	// R=
display[10] = 0x3d;

display[12] = 0x65;	// e=
display[13] = 0x3d;

display[16] = 0x35;	// Versionsnummer anzeigen (5)


display[17] = 0x55;		//U
display[18] = 0x3d;	

display[22] = 0x49;		//I
display[23] = 0x3d;

display[27] = 0x6e;		//n
display[28] = 0x3d;






for (;;) {;             /* loop forever */ // Bearbeitungszeit = 3315 s






sollvh = 0;

// ****** Ausgabe Enable an Wechselrichter *****************************

if ( PINC & (1<<6) ) enable=1; else enable=0;
if (enable) display[14] = 0x31; else display[14] = 0x30; 	// Ausgabe Enable an LCD

// *********Ausgabe Sollfrequenz an Wechselrichter***********************************

if (enable) sollvh=sollwert_f ; else sollvh=0;	// Sollwertsperre bei Enable =0

if (sollnh >0) richtung =0; else richtung =0x80;			// Bit liegt auf Bit 7 am Port D
if (richtung) display[11] = 0x31; else display[11] = 0x30; 	// Ausgabe Richtung an LCD


sollabs = sollnh;
if (sollnh <0) sollabs = (sollnh * -1);	// Bildung des Absolutwerts des Sollwerts
y=sollabs/10;display[3] =(y+0x30); display[4] =(sollabs-(y*10)+0x30);// Ausgabe LCD


y=sollabs;
for ( i = 0; i < 8; i++ )	// Sollwert drehen, zur Anpassung an die Hardware
{
   gespiegelt >>= 1;
   gespiegelt |= y & 0x80;
   y <<= 1;
}

PORTD=(((gespiegelt>>1)&0x7F)|richtung);


// ****** Ausgabe Sollspannung an Wechselrichter *****************************
// die Zuordnung Spannung zu Frequenz liegt in einer Tabelle 
spg=spannung[sollabs];
y=spg/10;display[7] =(y+0x30); display[8] =(spg-(y*10)+0x30);// Ausgabe LCD


y=spg;
for ( i = 0; i < 8; i++ )	// Sollwert drehen, zur Anpassung an die Hardware
{
   gespiegelt >>= 1;
   gespiegelt |= y & 0x80;
   y <<= 1;
}

PORTC= (((gespiegelt>>2)&0x3F) |enable);


j++;
// ****** Einlesen Istwert Spannung Uzk *********************************
ADMUX=0x00;		// PortA0 = ADC0 = Analog Eingang
      ADCSRA |= _BV(6);  // single conversion mode ein
      while(ADCSRA & (1<<6));  // warten bis Konvertierung abgeschlosen
      spannung_ist_sample+=ADCW;  // Aufsummierung der Samplewerte
  

// ****** Einlesen Istwert Strom *********************************
ADMUX=0x01;		// PortA1 = ADC1 = Analog Eingang
      ADCSRA |= _BV(6);  // single conversion mode ein
      while(ADCSRA & (1<<6));  // warten bis Konvertierung abgeschlosen
      strom_ist_sample+=ADCW;  // Aufsummierung der Samplewerte


// ****** Einlesen Sollwert  *********************************
ADMUX=0x02;		// PortA2 = ADC2 = Analog Eingang
      ADCSRA |= _BV(6);  // single conversion mode ein
      while(ADCSRA & (1<<6));  // warten bis Konvertierung abgeschlosen
      sollwert_f_sample+=ADCW;  // Aufsummierung der Samplewerte

/* Reserve Analogeingang**************************************************
// ****** Einlesen Sollwert  *********************************
ADMUX=0x03;		// PortA3 = ADC3 = Analog Eingang
      ADCSRA |= _BV(6);  // single conversion mode ein
      while(ADCSRA & (1<<6));  // warten bis Konvertierung abgeschlosen
      sollwert_u_sample+=ADCW;  // Aufsummierung der Samplewerte
**************************************************************************
*/

if (j>=20){
		strom_ist=strom_ist_sample/20;  // Aritmethisches Mittel der Samplewerte 
		spannung_ist=spannung_ist_sample/50;  // Aritmethisches Mittel der Samplewerte (auf 5V = 400 normiert)
												// gendert von 51 auf 50 wegen ungnstiger Toleranz der Spannungsteiler Widerstnde
		sollwert_f=sollwert_f_sample/334;  // Aritmethisches Mittel der Samplewerte und normiert auf 61
		//sollwert_u=sollwert_u_sample/1000;  // Aritmethisches Mittel der Samplewerte und normiert auf 20
		strom_ist_sample=0;  // Samplewerte auf 0
		spannung_ist_sample=0;  // Samplewerte auf 0
		sollwert_f_sample=0;  // Samplewerte auf 0
		//sollwert_u_sample=0;  // Samplewerte auf 0
		j=0;
		}


i=strom_ist; y=i/100; display[24] =(y+0x30); i-=(y*100);y=i/10; display[25] =(y+0x30); i-=(y*10); display[26] =(i+0x30);// Ausgabe LCD

i=spannung_ist; y=i/100; display[19] =(y+0x30); i-=(y*100);y=i/10; display[20] =(y+0x30); i-=(y*10); display[21] =(i+0x30);// Ausgabe LCD



// Ist-Drehzahlerfassung noch im Test!!!!!!!!!!!!!!!!!!!!!!!!!!!
if ( PINC & (1<<5) ) ntakt=1; else ntakt=0;

if ((ntakt) && (ntaktalt==0)) nistsample++;

ntaktalt=ntakt;

i=nist; y=i/100; display[29] =(y+0x30); i-=(y*100);y=i/10; display[30] =(y+0x30); i-=(y*10); display[31] =(i+0x30);// Ausgabe LCD
// Ist-Drehzahlerfassung noch im Test!!!!!!!!!!!!!!!!!!!!!!!!!!!




    }    	//Ende Loop for ever        
} 		//Ende Hauptprogramm      




